// Copyright (c) 2009, Willow Garage, Inc.
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of Willow Garage, Inc. nor the names of its
//       contributors may be used to endorse or promote products derived from
//       this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

#include "rosbag/message_instance.h"

using std::string;
using ros::Time;
using boost::shared_ptr;

namespace rosbag {

MessageInstance::MessageInstance(ConnectionInfo const* connection_info, IndexEntry const& index_entry, Bag const& bag) :
	connection_info_(connection_info), index_entry_(index_entry), bag_(&bag)
{
	init();
}

/*
MessageInstance::MessageInstance(const MessageInstance & m) :
	connection_info_(m.connection_info_), index_entry_(m.index_entry_), bag_(m.bag_), buffer_(m.buffer_),
	buffered_(m.buffered_), time_(m.time_)
{
}
*/

void MessageInstance::init()
{   /*
    time_ = index_entry_.time;
    topic_ = connection_info_->topic;
    data_type_ = connection_info_->datatype;
    md5_sum_ = connection_info_->md5sum;
    message_definition_ = connection_info_->msg_def;
    connection_header_ = connection_info_->header;
    caller_id_ = getCallerId_priv();
    is_latching_ = isLatching_priv();
    */
    size_ = bag_->readMessageDataSize(index_entry_);

}

void MessageInstance::buffer_message() {
    buffer_ = boost::shared_ptr<MessageBuffer>(new MessageBuffer());
    bag_->readMessageDataIntoStream(index_entry_, *buffer_);
}

Time const&   MessageInstance::getTime()              const { return index_entry_.time; }//time_; }
string const& MessageInstance::getTopic()             const { return connection_info_->topic; }//topic_; }
string const& MessageInstance::getDataType()          const { return connection_info_->datatype; }//data_type_; }
string const& MessageInstance::getMD5Sum()            const { return connection_info_->md5sum; } // md5_sum_;   }
string const& MessageInstance::getMessageDefinition() const { return connection_info_->msg_def; } //message_definition_; }

shared_ptr<ros::M_string> MessageInstance::getConnectionHeader() const { return connection_info_->header; } // connection_header_; }

//string MessageInstance::getCallerId_priv() const { return caller_id_; }
//bool MessageInstance::isLatching_priv() const { return is_latching_; }

string MessageInstance::getCallerId() const {
    ros::M_string::const_iterator header_iter = connection_info_->header->find("callerid");
    return header_iter != connection_info_->header->end() ? header_iter->second : string("");
}

bool MessageInstance::isLatching() const {
    ros::M_string::const_iterator header_iter = connection_info_->header->find("latching");
    return header_iter != connection_info_->header->end() && header_iter->second == "1";
}

uint32_t MessageInstance::size() const {
    return size_;
}

ros::AdvertiseOptions createAdvertiseOptions(MessageInstance const& m, uint32_t queue_size) {
    return ros::AdvertiseOptions(m.getTopic(), queue_size, m.getMD5Sum(), m.getDataType(), m.getMessageDefinition());
}

} // namespace rosbag
